package com.coldlightalchemist.notesapp;

import android.content.DialogInterface;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;

import com.coldlightalchemist.notesapp.db.NoteDB;
import com.coldlightalchemist.notesapp.db.entities.Note;
import com.google.android.material.snackbar.Snackbar;

public class NewNote extends AppCompatActivity {

    private EditText newNoteTitle;

    private EditText newNoteContent;

    private Button saveNote;

    private Note note;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_new_note);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        newNoteTitle = (EditText) findViewById(R.id.newNoteTitle);
        newNoteContent = (EditText) findViewById(R.id.newNoteContent);

        if(getIntent().hasExtra("editableNote")) {
            note = (Note) getIntent().getSerializableExtra("editableNote");

            newNoteTitle.getText().clear();
            newNoteTitle.getText().append(note.noteTitle);

            newNoteContent.getText().clear();
            newNoteContent.getText().append(note.noteContent);
        }

        saveNote = (Button) findViewById(R.id.saveNote);
        saveNote.setOnClickListener((v) -> {
            if(newNoteTitle.getText().toString().isEmpty() || newNoteContent.getText().toString().isEmpty()) {
                Snackbar.make(v, "You must have a title and content", Snackbar.LENGTH_LONG).show();
                return;
            }

            boolean noteWasNull = note == null;

            if(noteWasNull) {
                note = new Note();
            }

            note.noteTitle = newNoteTitle.getText().toString();
            note.noteContent = newNoteContent.getText().toString();

            if(noteWasNull) {
                NoteDB.execute(() -> {
                    NoteDB.getDB().noteDAO().insertAll(note);

                    NoteDB.getMainLoopHandler().post(() -> {
                        finish();
                    });
                });
            } else {
                NoteDB.execute(() -> {
                    NoteDB.getDB().noteDAO().update(note);

                    NoteDB.getMainLoopHandler().post(() -> {
                        finish();
                    });
                });

            }
        });

        saveNote.setOnLongClickListener((v) -> {
            AlertDialog.Builder alertBuilder = new AlertDialog.Builder(v.getContext());

            DialogInterface.OnClickListener listener = (d, w) -> {
                switch(w) {
                    case DialogInterface.BUTTON_POSITIVE:
                        if(note != null) {
                            NoteDB.execute(() -> {
                                NoteDB.getDB().noteDAO().delete(note);

                                NoteDB.getMainLoopHandler().post(() -> {
                                    finish();
                                });
                            });

                        }
                        break;
                    case DialogInterface.BUTTON_NEGATIVE:
                        break;
                }
            };

            alertBuilder.setMessage("Are you sure you want to delete this note?")
                    .setPositiveButton("Yes", listener)
                    .setNegativeButton("No", listener)
                    .show();

            return true;
        });
    }
}