package com.coldlightalchemist.notesapp;

import android.content.Intent;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.coldlightalchemist.notesapp.db.NoteDB;
import com.coldlightalchemist.notesapp.db.dao.NoteDAO;
import com.coldlightalchemist.notesapp.db.entities.Note;

import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.function.Consumer;

public class NotesRecyclerViewAdapter extends RecyclerView.Adapter<NotesRecyclerViewAdapter.NoteRecyclerViewHolder> {
    private final NoteDAO noteSource;

    private final Consumer<Intent> activityStarter;

    public static class NoteRecyclerViewHolder extends RecyclerView.ViewHolder {
        private final TextView rvNoteTitle;

        private Note note;

        public NoteRecyclerViewHolder(View itemView) {
            super(itemView);

            rvNoteTitle = itemView.findViewById(R.id.noteTitle);
        }

        public void setNote(Note note) {
            this.note = note;

            rvNoteTitle.setText(note.noteTitle);
        }

        public Note getNote() {
            return note;
        }
    }

    public NotesRecyclerViewAdapter(NoteDAO noteSource, Consumer<Intent> activityStarter) {
        this.noteSource = noteSource;
        this.activityStarter = activityStarter;
    }

    @NonNull
    @Override
    public NoteRecyclerViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.note_recycler_view_item, parent, false);

        NoteRecyclerViewHolder holder = new NoteRecyclerViewHolder(view);

        view.setOnClickListener((v) -> {
            Intent intent = new Intent(v.getContext(), NewNote.class);
            intent.putExtra("editableNote", holder.getNote());
            activityStarter.accept(intent);
        });

        return holder;
    }

    @Override
    public void onBindViewHolder(@NonNull NoteRecyclerViewHolder holder, int position) {
        NoteDB.execute(() -> {
            List<Note> notes = noteSource.getAll();

            NoteDB.getMainLoopHandler().post(() -> {
                holder.setNote(notes.get(position));
            });
        });
    }

    @Override
    public int getItemCount() {
        try {
            return NoteDB.execute(() -> {
                return NoteDB.getDB().noteDAO().getAll().size();
            }).get();
        } catch (ExecutionException | InterruptedException e) {
            // Cop out
            Log.d("ITEMCOUNTISSUE", e.toString());
            return 0;
        }
    }
}
