package com.coldlightalchemist.notesapp.db;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import androidx.room.Database;
import androidx.room.Room;
import androidx.room.RoomDatabase;

import com.coldlightalchemist.notesapp.db.dao.NoteDAO;
import com.coldlightalchemist.notesapp.db.entities.Note;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

@Database(entities = {Note.class}, version = 1)
public abstract class NoteDB extends RoomDatabase {
    public abstract NoteDAO noteDAO();

    private static NoteDB db;

    private static ExecutorService executorService;

    private static Handler handler;

    public static NoteDB getDB() {
        if (db == null) {
            throw new RuntimeException("DB not built");
        }

        return db;
    }

    public static NoteDB getDB(Context applicationContext) {
        if (db == null) {
            db = Room.databaseBuilder(applicationContext, NoteDB.class, "Notes")
                    .build();
        }

        return db;
    }

    public static Handler getMainLoopHandler() {
        if (handler == null) {
            handler = new Handler(Looper.getMainLooper());
        }

        return handler;
    }

    public static void execute(Runnable threadExecute) {
        if (executorService == null) {
            executorService = Executors.newSingleThreadExecutor();
        }

        executorService.execute(threadExecute);
    }

    public static <T> Future<T> execute(Callable<T> threadExecute) {
        if (executorService == null) {
            executorService = Executors.newSingleThreadExecutor();
        }

        return executorService.submit(threadExecute);
    }
}
